@@ -220,6 +220,12 @@ pub struct BuildOptions {
     #[arg(long)]
     pub zig: bool,
 
+    /// Include debug info files (.pdb on Windows, .dSYM on macOS, .dwp on Linux)
+    /// in the wheel. When enabled, maturin automatically configures
+    /// split-debuginfo=packed so that separate debug info files are produced.
+    #[arg(long)]
+    pub include_debuginfo: bool,
+
     /// Cargo build options
     #[command(flatten)]
     pub cargo: CargoOptions,
@@ -604,6 +610,7 @@ pub struct BuildContextBuilder {
     strip: Option<bool>,
     editable: bool,
     sdist_only: bool,
+    pyproject_toml_path: Option<PathBuf>,
 }
 
 impl BuildContextBuilder {
@@ -613,6 +620,7 @@ impl BuildContextBuilder {
             strip: None,
             editable: false,
             sdist_only: false,
+            pyproject_toml_path: None,
         }
     }
 
@@ -631,12 +639,18 @@ impl BuildContextBuilder {
         self
     }
 
+    pub fn pyproject_toml_path(mut self, path: Option<PathBuf>) -> Self {
+        self.pyproject_toml_path = path;
+        self
+    }
+
     pub fn build(self) -> Result<BuildContext> {
         let Self {
             build_options,
             strip,
             editable,
             sdist_only,
+            pyproject_toml_path: explicit_pyproject_path,
         } = self;
         build_options.compression.validate();
         let ProjectResolver {
@@ -654,9 +668,11 @@ impl BuildContextBuilder {
             build_options.manifest_path.clone(),
             build_options.cargo.clone(),
             editable,
+            explicit_pyproject_path,
         )?;
         let pyproject = pyproject_toml.as_ref();
 
+        let extra_pyo3_features = pyo3_features_from_conditional(pyproject);
         let bridge = find_bridge(
             &cargo_metadata,
             build_options.bindings.as_deref().or_else(|| {
@@ -667,6 +683,7 @@ impl BuildContextBuilder {
                     x.bindings()
                 })
             }),
+            &extra_pyo3_features,
         )?;
         debug!("Resolved bridge model: {:?}", bridge);
 
@@ -762,6 +779,10 @@ impl BuildContextBuilder {
         }
 
         let strip = strip.unwrap_or_else(|| pyproject.map(|x| x.strip()).unwrap_or_default());
+        if strip && build_options.include_debuginfo {
+            bail!("--include-debuginfo cannot be used with --strip");
+        }
+        let include_debuginfo = build_options.include_debuginfo;
         let skip_auditwheel = pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default()
             || build_options.skip_auditwheel;
         let auditwheel = build_options
@@ -896,19 +917,8 @@ impl BuildContextBuilder {
             pyproject_toml
                 .as_ref()
                 .and_then(|p| p.maturin())
-                .and_then(|m| m.features.as_ref())
-                .map(|specs| {
-                    specs
-                        .iter()
-                        .filter_map(|spec| match spec {
-                            FeatureSpec::Conditional {
-                                feature,
-                                python_version,
-                            } => Some((feature.clone(), python_version.clone())),
-                            FeatureSpec::Plain(_) => None,
-                        })
-                        .collect()
-                })
+                .and_then(|m| m.features.clone())
+                .map(|specs| FeatureSpec::split(specs).1)
                 .unwrap_or_default()
         } else {
             Vec::new()
@@ -940,6 +950,7 @@ impl BuildContextBuilder {
             pypi_validation,
             sbom,
             include_import_lib,
+            include_debuginfo,
             conditional_features,
         })
     }
@@ -1119,23 +1130,63 @@ fn filter_cargo_targets(
     Ok(targets)
 }
 
+/// Extract pyo3/pyo3-ffi feature names from conditional features in pyproject.toml.
+///
+/// For a conditional feature like `pyo3/abi3-py311`, this extracts `abi3-py311`
+/// for the corresponding binding crate.
+pub fn pyo3_features_from_conditional(
+    pyproject: Option<&crate::PyProjectToml>,
+) -> HashMap<&'static str, Vec<String>> {
+    let mut extra: HashMap<&'static str, Vec<String>> = HashMap::new();
+    let features = match pyproject
+        .and_then(|p| p.maturin())
+        .and_then(|m| m.features.as_ref())
+    {
+        Some(f) => f,
+        None => return extra,
+    };
+    let crate_names: &[&'static str] = &["pyo3", "pyo3-ffi"];
+    for spec in features {
+        let feature = match spec {
+            FeatureSpec::Plain(_) => continue,
+            FeatureSpec::Conditional { feature, .. } => feature,
+        };
+        for &crate_name in crate_names {
+            let prefix = format!("{crate_name}/");
+            if let Some(feat_name) = feature.strip_prefix(&prefix) {
+                extra
+                    .entry(crate_name)
+                    .or_default()
+                    .push(feat_name.to_string());
+            }
+        }
+    }
+    extra
+}
+
 /// pyo3 supports building abi3 wheels if the unstable-api feature is not selected
-fn has_abi3(deps: &HashMap<&str, &Node>) -> Result<Option<Abi3Version>> {
+fn has_abi3(
+    deps: &HashMap<&str, &Node>,
+    extra_features: &HashMap<&str, Vec<String>>,
+) -> Result<Option<Abi3Version>> {
     for &lib in PYO3_BINDING_CRATES.iter() {
         let lib = lib.as_str();
         if let Some(&pyo3_crate) = deps.get(lib) {
+            let extra = extra_features.get(lib);
             // Find the minimal abi3 python version. If there is none, abi3 hasn't been selected
             // This parser abi3-py{major}{minor} and returns the minimal (major, minor) tuple
-            let abi3_selected = pyo3_crate
+            let all_features: Vec<&str> = pyo3_crate
                 .features
                 .iter()
                 .map(AsRef::as_ref)
-                .any(|x| x == "abi3");
+                .chain(extra.into_iter().flatten().map(String::as_str))
+                .collect();
 
-            let min_abi3_version = pyo3_crate
-                .features
+            let abi3_selected = all_features.contains(&"abi3");
+
+            let min_abi3_version = all_features
                 .iter()
-                .filter(|&x| x.starts_with("abi3-py") && x.len() >= "abi3-pyxx".len())
+                .filter(|&&x| x.starts_with("abi3-py") && x.len() >= "abi3-pyxx".len())
                 .map(|x| {
                     Ok((
                         (x.as_bytes()[7] as char).to_string().parse::<u8>()?,
@@ -1270,7 +1321,11 @@ fn current_crate_dependencies(cargo_metadata: &Metadata) -> Result<HashMap<&str,
 }
 
 /// Tries to determine the [BridgeModel] for the target crate
-pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<BridgeModel> {
+pub fn find_bridge(
+    cargo_metadata: &Metadata,
+    bridge: Option<&str>,
+    extra_pyo3_features: &HashMap<&str, Vec<String>>,
+) -> Result<BridgeModel> {
     let deps = current_crate_dependencies(cargo_metadata)?;
     let packages: HashMap<&str, &cargo_metadata::Package> = cargo_metadata
         .packages
@@ -1368,7 +1423,7 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
                 }
             }
 
-            return if let Some(abi3_version) = has_abi3(&deps)? {
+            return if let Some(abi3_version) = has_abi3(&deps, extra_pyo3_features)? {
                 eprintln!("🔗 Found {lib} bindings with abi3 support");
                 let pyo3 = bridge.pyo3().expect("should be pyo3 bindings");
                 let bindings = PyO3 {
@@ -1754,13 +1809,8 @@ impl CargoOptions {
         if let Some(feature_specs) = tool_maturin.features
             && self.features.is_empty()
         {
-            self.features = feature_specs
-                .into_iter()
-                .filter_map(|spec| match spec {
-                    FeatureSpec::Plain(f) => Some(f),
-                    FeatureSpec::Conditional { .. } => None,
-                })
-                .collect();
+            let (plain, _conditional) = FeatureSpec::split(feature_specs);
+            self.features = plain;
             args_from_pyproject.push("features");
         }
 
@@ -1825,12 +1875,13 @@ mod tests {
             .exec()
             .unwrap();
 
+        let no_extra = HashMap::new();
         assert!(matches!(
-            find_bridge(&pyo3_mixed, None),
+            find_bridge(&pyo3_mixed, None, &no_extra),
             Ok(BridgeModel::PyO3 { .. })
         ));
         assert!(matches!(
-            find_bridge(&pyo3_mixed, Some("pyo3")),
+            find_bridge(&pyo3_mixed, Some("pyo3"), &no_extra),
             Ok(BridgeModel::PyO3 { .. })
         ));
     }
@@ -1859,8 +1910,12 @@ mod tests {
                 },
             }),
         });
-        assert_eq!(find_bridge(&pyo3_pure, None).unwrap(), bridge);
-        assert_eq!(find_bridge(&pyo3_pure, Some("pyo3")).unwrap(), bridge);
+        let no_extra = HashMap::new();
+        assert_eq!(find_bridge(&pyo3_pure, None, &no_extra).unwrap(), bridge);
+        assert_eq!(
+            find_bridge(&pyo3_pure, Some("pyo3"), &no_extra).unwrap(),
+            bridge
+        );
     }
 
     #[test]
@@ -1870,7 +1925,8 @@ mod tests {
             .exec()
             .unwrap();
 
-        assert!(find_bridge(&pyo3_pure, None).is_err());
+        let no_extra = HashMap::new();
+        assert!(find_bridge(&pyo3_pure, None, &no_extra).is_err());
 
         let pyo3_pure = MetadataCommand::new()
             .manifest_path(Path::new("test-crates/pyo3-feature").join("Cargo.toml"))
@@ -1879,7 +1935,7 @@ mod tests {
             .unwrap();
 
         assert!(matches!(
-            find_bridge(&pyo3_pure, None).unwrap(),
+            find_bridge(&pyo3_pure, None, &no_extra).unwrap(),
             BridgeModel::PyO3 { .. }
         ));
     }
@@ -1891,13 +1947,17 @@ mod tests {
             .exec()
             .unwrap();
 
+        let no_extra = HashMap::new();
         assert_eq!(
-            find_bridge(&cffi_pure, Some("cffi")).unwrap(),
+            find_bridge(&cffi_pure, Some("cffi"), &no_extra).unwrap(),
+            BridgeModel::Cffi
+        );
+        assert_eq!(
+            find_bridge(&cffi_pure, None, &no_extra).unwrap(),
             BridgeModel::Cffi
         );
-        assert_eq!(find_bridge(&cffi_pure, None).unwrap(), BridgeModel::Cffi);
 
-        assert!(find_bridge(&cffi_pure, Some("pyo3")).is_err());
+        assert!(find_bridge(&cffi_pure, Some("pyo3"), &no_extra).is_err());
     }
 
     #[test]
@@ -1907,27 +1967,28 @@ mod tests {
             .exec()
             .unwrap();
 
+        let no_extra = HashMap::new();
         assert_eq!(
-            find_bridge(&hello_world, Some("bin")).unwrap(),
+            find_bridge(&hello_world, Some("bin"), &no_extra).unwrap(),
             BridgeModel::Bin(None)
         );
         assert_eq!(
-            find_bridge(&hello_world, None).unwrap(),
+            find_bridge(&hello_world, None, &no_extra).unwrap(),
             BridgeModel::Bin(None)
         );
 
-        assert!(find_bridge(&hello_world, Some("pyo3")).is_err());
+        assert!(find_bridge(&hello_world, Some("pyo3"), &no_extra).is_err());
 
         let pyo3_bin = MetadataCommand::new()
             .manifest_path(Path::new("test-crates/pyo3-bin").join("Cargo.toml"))
             .exec()
             .unwrap();
         assert!(matches!(
-            find_bridge(&pyo3_bin, Some("bin")).unwrap(),
+            find_bridge(&pyo3_bin, Some("bin"), &no_extra).unwrap(),
             BridgeModel::Bin(Some(_))
         ));
         assert!(matches!(
-            find_bridge(&pyo3_bin, None).unwrap(),
+            find_bridge(&pyo3_bin, None, &no_extra).unwrap(),
             BridgeModel::Bin(Some(_))
         ));
     }
