@@ -170,8 +170,8 @@ pub fn write_python_part(
         }
         let relative = absolute.strip_prefix(python_dir).unwrap();
         if !absolute.is_dir() {
-            if is_develop_native_library(relative, &project_layout.extension_name) {
-                debug!("Ignoring native library {}", relative.display());
+            if is_develop_build_artifact(relative, &project_layout.extension_name) {
+                debug!("Ignoring develop build artifact {}", relative.display());
                 continue;
             }
             #[cfg(unix)]
@@ -383,29 +383,50 @@ pub fn write_pth(
     Ok(())
 }
 
-/// Check if a file is a native library artifact left behind by `maturin develop`.
+/// Check if a file is a build artifact left behind by `maturin develop`.
 ///
 /// `maturin develop` copies compiled extension modules (`.so`, `.pyd`, `.dll`, `.dylib`)
-/// directly into the Python source tree for editable installs. When `maturin build` later
-/// walks the same source tree to collect files for the wheel, these artifacts must be
-/// skipped to avoid conflicts with the freshly compiled library being added to the wheel.
+/// and their associated debug info files (`.pdb`, `.dSYM`, `.dwp`) directly into the
+/// Python source tree for editable installs. When `maturin build` later walks the same
+/// source tree to collect files for the wheel, these artifacts must be skipped to avoid
+/// conflicts with the freshly compiled library being added to the wheel.
 ///
-/// The artifacts follow different naming conventions depending on the binding type:
+/// The native library artifacts follow different naming conventions depending on the
+/// binding type:
 /// - **PyO3/pyo3-ffi**: `{ext_name}.cpython-3XX-*.so`, `{ext_name}.abi3.so`, `{ext_name}.pyd`
 /// - **CFFI**: `lib{ext_name}.so`, `lib{ext_name}.dylib` (Unix), `{ext_name}.dll` (Windows)
 /// - **UniFFI**: `lib{ext_name}.so`, `lib{ext_name}.dylib` (Unix), `{ext_name}.dll` (Windows)
-fn is_develop_native_library(relative_path: &Path, extension_name: &str) -> bool {
+///
+/// Debug info files (`.pdb`, `.dwp`, or files inside `.dSYM` bundles) are also excluded
+/// when their name matches the extension name, since they are re-added from the fresh
+/// build output when appropriate.
+fn is_develop_build_artifact(relative_path: &Path, extension_name: &str) -> bool {
     let Some(file_name) = relative_path.file_name() else {
         return false;
     };
     let file_name = file_name.to_string_lossy();
+
+    // Files inside a .dSYM bundle (macOS debug info directory) — match on the
+    // bundle directory name rather than the leaf filename (which can be
+    // Info.plist, a DWARF data file, etc.)
+    let dsym_bundle = relative_path
+        .components()
+        .map(|c| c.as_os_str().to_string_lossy())
+        .find(|c| c.ends_with(".dSYM"));
+    if let Some(bundle) = dsym_bundle {
+        let bundle = bundle.trim_end_matches(".dSYM");
+        return bundle.starts_with(extension_name)
+            || bundle.starts_with(&format!("lib{extension_name}"));
+    }
+
     let is_native_ext = file_name.ends_with(".so")
         || file_name.ends_with(".pyd")
         || file_name.ends_with(".dll")
         || file_name.ends_with(".dylib");
-    is_native_ext
-        && (file_name.starts_with(extension_name)
-            || file_name.starts_with(&format!("lib{extension_name}")))
+    let is_debuginfo = file_name.ends_with(".pdb") || file_name.ends_with(".dwp");
+    let name_matches = file_name.starts_with(extension_name)
+        || file_name.starts_with(&format!("lib{extension_name}"));
+    (is_native_ext || is_debuginfo) && name_matches
 }
 
 fn expand_compressed_tag(tag: &str) -> impl Iterator<Item = String> + '_ {
