@@ -10,7 +10,7 @@ use std::collections::BTreeSet;
 use std::io::Read;
 use std::path::{Path, PathBuf};
 use tar::Archive;
-use time::OffsetDateTime;
+use time::PrimitiveDateTime;
 use zip::ZipArchive;
 
 pub fn copy_dir_recursive(src: &Path, dst: &Path) -> std::io::Result<()> {
@@ -291,11 +291,11 @@ fn build_wheel_files(package: impl AsRef<Path>, unique_name: &str) -> Result<Zip
 
 pub fn check_wheel_mtimes(
     package: impl AsRef<Path>,
-    expected_mtime: Vec<OffsetDateTime>,
+    expected_mtime: Vec<PrimitiveDateTime>,
     unique_name: &str,
 ) -> Result<()> {
     let mut wheel = build_wheel_files(package, unique_name)?;
-    let mut mtimes = BTreeSet::<OffsetDateTime>::new();
+    let mut mtimes = BTreeSet::<PrimitiveDateTime>::new();
 
     for idx in 0..wheel.len() {
         let mtime = wheel.by_index(idx)?.last_modified().unwrap().try_into()?;
@@ -327,7 +327,8 @@ pub fn check_wheel_files(
 ) -> Result<()> {
     let wheel = build_wheel_files(package, unique_name)?;
     let drop_platform_specific_files = |file: &&str| -> bool {
-        !matches!(Path::new(file).extension(), Some(ext) if ext == "pyc" || ext == "pyd" || ext == "so")
+        !matches!(Path::new(file).extension(), Some(ext) if ext == "pyc" || ext == "pyd" || ext == "so" || ext == "pdb" || ext == "dwp")
+            && !file.contains(".dSYM/")
     };
     assert_eq!(
         wheel
@@ -561,7 +562,7 @@ pub fn test_build_wheels_from_sdist(package: impl AsRef<Path>, unique_name: &str
         .context("Failed to build source distribution")?;
 
     // Step 2: Unpack sdist and build wheels from it
-    let (_tmp, cargo_toml) = unpack_sdist(&sdist_path)?;
+    let (_tmp, cargo_toml, pyproject_toml) = unpack_sdist(&sdist_path)?;
     let wheel_options = BuildOptions {
         out: Some(wheel_dir),
         cargo: CargoOptions {
@@ -578,6 +579,7 @@ pub fn test_build_wheels_from_sdist(package: impl AsRef<Path>, unique_name: &str
         .into_build_context()
         .strip(Some(cfg!(feature = "faster-tests")))
         .editable(false)
+        .pyproject_toml_path(Some(pyproject_toml))
         .build()?;
     let wheels = wheel_context.build_wheels()?;
     assert!(
